Method: Beaker::Hypervisor#configure

Defined in:
lib/beaker/hypervisor.rb

#configure(opts = {}) ⇒ Object

Default configuration steps to be run for a given hypervisor. Any additional configuration to be done to the provided SUT for test execution to be successful.



84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
# File 'lib/beaker/hypervisor.rb', line 84

def configure(opts = {})
  begin
    return unless @options[:configure]
    run_in_parallel = run_in_parallel? opts, @options, 'configure'
    block_on @hosts, { :run_in_parallel => run_in_parallel} do |host|
      if host[:timesync]
        timesync(host, @options)
      end
    end
    if @options[:root_keys]
      sync_root_keys(@hosts, @options)
    end
    if @options[:add_el_extras]
      add_el_extras(@hosts, @options)
    end
    if @options[:disable_iptables]
      disable_iptables @hosts, @options
    end
    if @options[:set_env]
      set_env(@hosts, @options)
    end
    if @options[:disable_updates]
      disable_updates(@hosts, @options)
    end
  rescue SignalException => ex
    if ex.signo == 15 #SIGTERM
      report_and_raise(@logger, ex, "configure")
    end
    raise
  end
end