Method: Beaker::Host#node_name
- Defined in:
- lib/beaker/host.rb
#node_name ⇒ Object
84 85 86 87 88 89 |
# File 'lib/beaker/host.rb', line 84 def node_name # TODO: might want to consider caching here; not doing it for now because # I haven't thought through all of the possible scenarios that could # cause the value to change after it had been cached. result = puppet_configprint['node_name_value'].strip end |