Module: Beaker::HostPrebuiltSteps
Overview
Provides convienience methods for commonly run actions on hosts
Constant Summary collapse
- NTPSERVER =
'pool.ntp.org'- SLEEPWAIT =
5- TRIES =
5- UNIX_PACKAGES =
['curl', 'ntpdate']
- FREEBSD_PACKAGES =
['curl', 'perl5']
- OPENBSD_PACKAGES =
['curl']
- WINDOWS_PACKAGES =
['curl']
- PSWINDOWS_PACKAGES =
[]
- SLES10_PACKAGES =
['curl']
- SLES_PACKAGES =
['curl', 'ntp']
- DEBIAN_PACKAGES =
['curl', 'ntpdate', 'lsb-release']
- CUMULUS_PACKAGES =
['curl', 'ntpdate']
- ETC_HOSTS_PATH =
"/etc/hosts"- ETC_HOSTS_PATH_SOLARIS =
"/etc/inet/hosts"- ROOT_KEYS_SCRIPT =
"https://raw.githubusercontent.com/puppetlabs/puppetlabs-sshkeys/master/templates/scripts/manage_root_authorized_keys"- ROOT_KEYS_SYNC_CMD =
"curl -k -o - -L #{ROOT_KEYS_SCRIPT} | %s"- APT_CFG =
%q{ Acquire::http::Proxy "http://proxy.puppetlabs.net:3128/"; }- IPS_PKG_REPO =
"http://solaris-11-internal-repo.delivery.puppetlabs.net"
Instance Method Summary collapse
-
#add_el_extras(host, opts) ⇒ Object
Install EPEL on host or hosts with platform = /el-(5|6)/.
-
#additive_hash_merge(h1, h2) ⇒ Hash
Merge the two provided hashes so that an array of values is created from collisions.
-
#apt_get_update(hosts) ⇒ Object
Run 'apt-get update' on the provided host or hosts.
-
#check_and_install_packages_if_needed(host, package_list) ⇒ Object
Installs the given packages if they aren't already on a host.
-
#construct_env(host, opts) ⇒ Hash
Create the hash of default environment from host (:host_env), global options hash (:host_env) and default PE/Foss puppet variables.
-
#copy_file_to_remote(host, file_path, file_content) ⇒ Object
Create a file on host or hosts at the provided file path with the provided file contents.
-
#copy_ssh_to_root(host, opts) ⇒ Object
Make it possible to log in as root by copying the current users ssh keys to the root account.
-
#disable_iptables(host, opts) ⇒ Object
Disable iptables on centos, does nothing on other platforms.
-
#disable_se_linux(host, opts) ⇒ Object
Disable SELinux on centos, does nothing on other platforms.
-
#enable_root_login(host, opts) ⇒ Object
Update sshd_config on debian, ubuntu, centos, el, redhat, cumulus, and fedora boxes to allow for root login.
-
#epel_info_for(host, opts) ⇒ String
Determine the Extra Packages for Enterprise Linux URL for the provided Enterprise Linux host.
-
#get_domain_name(host) ⇒ Object
Determine the domain name of the provided host from its /etc/resolv.conf.
- #get_ip(host) ⇒ Object deprecated Deprecated.
-
#hack_etc_hosts(hosts, opts) ⇒ Object
Update /etc/hosts to make it possible for each provided host to reach each other host by name.
-
#package_proxy(host, opts) ⇒ Object
Setup files for enabling requests to pass to a proxy server This works for the APT package manager on debian, ubuntu, and cumulus and YUM package manager on el, centos, fedora and redhat.
-
#proxy_config(host, opts) ⇒ Object
On ubuntu, debian, or cumulus host or hosts: alter apt configuration to use the internal Puppet Labs proxy APT_CFG proxy.
-
#set_env(host, opts) ⇒ Object
Add a host specific set of env vars to each provided host's ~/.ssh/environment.
-
#set_etc_hosts(host, etc_hosts) ⇒ Object
Append the provided string to the /etc/hosts file of the provided host.
-
#sync_root_keys(host, opts) ⇒ Object
Install a set of authorized keys using ROOT_KEYS_SCRIPT.
-
#timesync(host, opts) ⇒ Object
Run timesync on the provided hosts.
-
#validate_host(host, opts) ⇒ Object
Validate that hosts are prepared to be used as SUTs, if packages are missing attempt to install them.
Methods included from DSL::Patterns
Instance Method Details
#add_el_extras(host, opts) ⇒ Object
Install EPEL on host or hosts with platform = /el-(5|6)/. Do nothing on host or hosts of other platforms.
247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 247 def add_el_extras( host, opts ) #add_el_extras #only supports el-* platforms logger = opts[:logger] debug_opt = opts[:debug] ? 'vh' : '' block_on host do |host| case when el_based?(host) && ['5','6'].include?(host['platform'].version) result = host.exec(Command.new('rpm -qa | grep epel-release'), :acceptable_exit_codes => [0,1]) if result.exit_code == 1 url, arch, pkg = epel_info_for host, opts host.exec(Command.new("rpm -i#{debug_opt} #{url}/#{arch}/#{pkg}")) #update /etc/yum.repos.d/epel.repo for new baseurl host.exec(Command.new("sed -i -e 's;#baseurl.*$;baseurl=#{Regexp.escape(url)}/\$basearch;' /etc/yum.repos.d/epel.repo")) #remove mirrorlist host.exec(Command.new("sed -i -e '/mirrorlist/d' /etc/yum.repos.d/epel.repo")) host.exec(Command.new('yum clean all && yum makecache')) end else logger.debug "#{host}: package repo configuration not modified" end end rescue => e report_and_raise(logger, e, "add_repos") end |
#additive_hash_merge(h1, h2) ⇒ Hash
Merge the two provided hashes so that an array of values is created from collisions
453 454 455 456 457 458 459 460 461 462 463 464 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 453 def additive_hash_merge h1, h2 merged_hash = {} normalized_h2 = h2.inject({}) { |h, (k, v)| h[k.to_s.upcase] = v; h } h1.each_pair do |key, val| normalized_key = key.to_s.upcase if normalized_h2.has_key?(normalized_key) merged_hash[key] = [h1[key], normalized_h2[normalized_key]] merged_hash[key] = merged_hash[key].uniq #remove dupes end end merged_hash end |
#apt_get_update(hosts) ⇒ Object
Run 'apt-get update' on the provided host or hosts. If the platform of the provided host is not ubuntu, debian or cumulus: do nothing.
186 187 188 189 190 191 192 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 186 def apt_get_update hosts block_on hosts do |host| if host[:platform] =~ /ubuntu|debian|cumulus/ host.exec(Command.new("apt-get update")) end end end |
#check_and_install_packages_if_needed(host, package_list) ⇒ Object
Installs the given packages if they aren't already on a host
121 122 123 124 125 126 127 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 121 def check_and_install_packages_if_needed host, package_list package_list.each do |pkg| if not host.check_for_package pkg host.install_package pkg end end end |
#construct_env(host, opts) ⇒ Hash
Create the hash of default environment from host (:host_env), global options hash (:host_env) and default PE/Foss puppet variables
470 471 472 473 474 475 476 477 478 479 480 481 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 470 def construct_env host, opts env = additive_hash_merge(host[:host_env], opts[:host_env]) env.each_key do |key| separator = host['pathseparator'] if key == 'PATH' && (not host.is_powershell?) separator = ':' end env[key] = env[key].join(separator) end env end |
#copy_file_to_remote(host, file_path, file_content) ⇒ Object
Create a file on host or hosts at the provided file path with the provided file contents.
198 199 200 201 202 203 204 205 206 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 198 def copy_file_to_remote(host, file_path, file_content) block_on host do |host| Tempfile.open 'beaker' do |tempfile| File.open(tempfile.path, 'w') {|file| file.puts file_content } host.do_scp_to(tempfile.path, file_path, ) end end end |
#copy_ssh_to_root(host, opts) ⇒ Object
Make it possible to log in as root by copying the current users ssh keys to the root account
312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 312 def copy_ssh_to_root host, opts logger = opts[:logger] block_on host do |host| logger.debug "Give root a copy of current user's keys, on #{host.name}" if host['platform'] =~ /windows/ and host.is_cygwin? host.exec(Command.new('cp -r .ssh /cygdrive/c/Users/Administrator/.')) host.exec(Command.new('chown -R Administrator /cygdrive/c/Users/Administrator/.ssh')) elsif host['platform'] =~ /windows/ and not host.is_cygwin? host.exec(Command.new("if exist .ssh (xcopy .ssh C:\\Users\\Administrator\\.ssh /s /e)")) elsif host['platform'] =~ /osx/ host.exec(Command.new('sudo cp -r .ssh /var/root/.'), {:pty => true}) elsif host['platform'] =~ /freebsd/ host.exec(Command.new('sudo cp -r .ssh /root/.'), {:pty => true}) elsif host['platform'] =~ /openbsd/ host.exec(Command.new('sudo cp -r .ssh /root/.'), {:pty => true}) else host.exec(Command.new('sudo su -c "cp -r .ssh /root/."'), {:pty => true}) end end end |
#disable_iptables(host, opts) ⇒ Object
Disable iptables on centos, does nothing on other platforms
409 410 411 412 413 414 415 416 417 418 419 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 409 def disable_iptables host, opts logger = opts[:logger] block_on host do |host| if host['platform'] =~ /centos|el-|redhat|fedora|eos/ logger.debug("Disabling iptables on #{host.name}") host.exec(Command.new("sudo su -c \"/etc/init.d/iptables stop\""), {:pty => true}) else logger.warn("Attempting to disable iptables on non-supported platform: #{host.name}: #{host['platform']}") end end end |
#disable_se_linux(host, opts) ⇒ Object
Disable SELinux on centos, does nothing on other platforms
393 394 395 396 397 398 399 400 401 402 403 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 393 def disable_se_linux host, opts logger = opts[:logger] block_on host do |host| if host['platform'] =~ /centos|el-|redhat|fedora|eos/ @logger.debug("Disabling se_linux on #{host.name}") host.exec(Command.new("sudo su -c \"setenforce 0\""), {:pty => true}) else @logger.warn("Attempting to disable SELinux on non-supported platform: #{host.name}: #{host['platform']}") end end end |
#enable_root_login(host, opts) ⇒ Object
Update sshd_config on debian, ubuntu, centos, el, redhat, cumulus, and fedora boxes to allow for root login
Does nothing on other platfoms.
358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 358 def enable_root_login host, opts logger = opts[:logger] block_on host do |host| logger.debug "Update /etc/ssh/sshd_config to allow root login" if host['platform'] =~ /osx/ host.exec(Command.new("sudo sed -i '' 's/#PermitRootLogin no/PermitRootLogin Yes/g' /etc/sshd_config")) host.exec(Command.new("sudo sed -i '' 's/#PermitRootLogin yes/PermitRootLogin Yes/g' /etc/sshd_config")) elsif host['platform'] =~ /freebsd/ host.exec(Command.new("sudo sed -i -e 's/#PermitRootLogin no/PermitRootLogin yes/g' /etc/ssh/sshd_config"), {:pty => true} ) elsif host['platform'] =~ /openbsd/ host.exec(Command.new("sudo perl -pi -e 's/^PermitRootLogin no/PermitRootLogin yes/' /etc/ssh/sshd_config"), {:pty => true} ) elsif not host.is_powershell? host.exec(Command.new("sudo su -c \"sed -ri 's/^#?PermitRootLogin no|^#?PermitRootLogin yes/PermitRootLogin yes/' /etc/ssh/sshd_config\""), {:pty => true}) else logger.warn("Attempting to enable root login non-supported platform: #{host.name}: #{host['platform']}") end #restart sshd if host['platform'] =~ /debian|ubuntu|cumulus/ host.exec(Command.new("sudo su -c \"service ssh restart\""), {:pty => true}) elsif host['platform'] =~ /centos-7|el-7|redhat-7/ host.exec(Command.new("sudo -E systemctl restart sshd.service"), {:pty => true}) elsif host['platform'] =~ /centos|el-|redhat|fedora|eos/ host.exec(Command.new("sudo -E /sbin/service sshd reload"), {:pty => true}) elsif host['platform'] =~ /(free|open)bsd/ host.exec(Command.new("sudo /etc/rc.d/sshd restart")) else logger.warn("Attempting to update ssh on non-supported platform: #{host.name}: #{host['platform']}") end end end |
#epel_info_for(host, opts) ⇒ String
Determine the Extra Packages for Enterprise Linux URL for the provided Enterprise Linux host.
164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 164 def epel_info_for host, opts if !el_based?(host) raise "epel_info_for! not available for #{host.name} on platform #{host['platform']}" end version = host['platform'].version if version == '6' url = "#{host[:epel_url] || opts[:epel_url]}/#{version}" pkg = host[:epel_pkg] || opts[:epel_6_pkg] elsif version == '5' url = "#{host[:epel_url] || opts[:epel_url]}/#{version}" pkg = host[:epel_pkg] || opts[:epel_5_pkg] else raise "epel_info_for does not support el version #{version}, on #{host.name}" end return url, host[:epel_arch] || opts[:epel_arch] || 'i386', pkg end |
#get_domain_name(host) ⇒ Object
Determine the domain name of the provided host from its /etc/resolv.conf
275 276 277 278 279 280 281 282 283 284 285 286 287 288 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 275 def get_domain_name(host) domain = nil search = nil resolv_conf = host.exec(Command.new("cat /etc/resolv.conf")).stdout resolv_conf.each_line { |line| if line =~ /^\s*domain\s+(\S+)/ domain = $1 elsif line =~ /^\s*search\s+(\S+)/ search = $1 end } return domain if domain return search if search end |
#get_ip(host) ⇒ Object
Determine the ip address of the provided host
293 294 295 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 293 def get_ip(host) host.get_ip end |
#hack_etc_hosts(hosts, opts) ⇒ Object
Update /etc/hosts to make it possible for each provided host to reach each other host by name. Assumes that each provided host has host set; in the instance where a provider sets host to an address which facilitates access to the host externally, but the actual host addresses differ from this, we check first for the presence of a host key first, and use that if present.
341 342 343 344 345 346 347 348 349 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 341 def hack_etc_hosts hosts, opts etc_hosts = "127.0.0.1\tlocalhost localhost.localdomain\n" hosts.each do |host| etc_hosts += "#{host['vm_ip'] || host['ip'].to_s}\t#{host[:vmhostname] || host.name}\n" end hosts.each do |host| set_etc_hosts(host, etc_hosts) end end |
#package_proxy(host, opts) ⇒ Object
Setup files for enabling requests to pass to a proxy server This works for the APT package manager on debian, ubuntu, and cumulus and YUM package manager on el, centos, fedora and redhat.
427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 427 def package_proxy host, opts logger = opts[:logger] block_on host do |host| logger.debug("enabling proxy support on #{host.name}") case host['platform'] when /ubuntu/, /debian/, /cumulus/ host.exec(Command.new("echo 'Acquire::http::Proxy \"#{opts[:package_proxy]}/\";' >> /etc/apt/apt.conf.d/10proxy")) when /^el-/, /centos/, /fedora/, /redhat/, /eos/ host.exec(Command.new("echo 'proxy=#{opts[:package_proxy]}/' >> /etc/yum.conf")) else logger.debug("Attempting to enable package manager proxy support on non-supported platform: #{host.name}: #{host['platform']}") end end end |
#proxy_config(host, opts) ⇒ Object
On ubuntu, debian, or cumulus host or hosts: alter apt configuration to use the internal Puppet Labs proxy APT_CFG proxy. On solaris-11 host or hosts: alter pkg to point to the internal Puppet Labs proxy IPS_PKG_REPO.
Do nothing for other platform host or hosts.
218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 218 def proxy_config( host, opts ) logger = opts[:logger] block_on host do |host| case when host['platform'] =~ /ubuntu|debian|cumulus/ host.exec(Command.new("if test -f /etc/apt/apt.conf; then mv /etc/apt/apt.conf /etc/apt/apt.conf.bk; fi")) copy_file_to_remote(host, '/etc/apt/apt.conf', APT_CFG) apt_get_update(host) when host['platform'] =~ /solaris-11/ host.exec(Command.new("/usr/bin/pkg unset-publisher solaris || :")) host.exec(Command.new("/usr/bin/pkg set-publisher -g %s solaris" % IPS_PKG_REPO)) else logger.debug "#{host}: repo proxy configuration not modified" end end rescue => e report_and_raise(logger, e, "proxy_config") end |
#set_env(host, opts) ⇒ Object
Add a host specific set of env vars to each provided host's ~/.ssh/environment
486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 486 def set_env host, opts logger = opts[:logger] block_on host do |host| env = construct_env(host, opts) logger.debug("setting local environment on #{host.name}") case host['platform'] when /windows/ if host.is_cygwin? host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/sshd_config")) # we get periodic failures to restart the service, so looping these with re-attempts repeat_fibonacci_style_for(5) do 0 == host.exec(Command.new("cygrunsrv -E sshd"), :acceptable_exit_codes => [0, 1] ).exit_code end repeat_fibonacci_style_for(5) do 0 == host.exec(Command.new("cygrunsrv -S sshd"), :acceptable_exit_codes => [0, 1] ).exit_code end env['CYGWIN'] = 'nodosfilewarning' else #nothing to do here end when /osx/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/sshd_config")) host.exec(Command.new("launchctl unload /System/Library/LaunchDaemons/ssh.plist")) host.exec(Command.new("launchctl load /System/Library/LaunchDaemons/ssh.plist")) when /debian|ubuntu|cumulus/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("service ssh restart")) when /el-7|centos-7|redhat-7|oracle-7|scientific-7|eos-7/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("systemctl restart sshd.service")) when /el-|centos|fedora|redhat|oracle|scientific|eos/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("/sbin/service sshd restart")) when /sles/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("rcsshd restart")) when /solaris/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("svcadm restart svc:/network/ssh:default")) when /aix/ host.exec(Command.new("echo '\nPermitUserEnvironment yes' >> /etc/ssh/sshd_config")) host.exec(Command.new("stopsrc -g ssh")) host.exec(Command.new("startsrc -g ssh")) when /(free|open)bsd/ host.exec(Command.new("sudo perl -pi -e 's/^#?PermitUserEnvironment no/PermitUserEnvironment yes/' /etc/ssh/sshd_config"), {:pty => true} ) host.exec(Command.new("sudo /etc/rc.d/sshd restart")) end if not host.is_powershell? #ensure that ~/.ssh/environment exists host.exec(Command.new("mkdir -p #{Pathname.new(host[:ssh_env_file]).dirname}")) host.exec(Command.new("chmod 0600 #{Pathname.new(host[:ssh_env_file]).dirname}")) host.exec(Command.new("touch #{host[:ssh_env_file]}")) #add the constructed env vars to this host host.add_env_var('PATH', '$PATH') # FIXME if host['platform'] =~ /openbsd-(\d)\.?(\d)-(.+)/ version = "#{$1}.#{$2}" arch = $3 arch = 'amd64' if ['x64', 'x86_64'].include?(arch) host.add_env_var('PKG_PATH', "http://ftp.openbsd.org/pub/OpenBSD/#{version}/packages/#{arch}/") end end #add the env var set to this test host env.each_pair do |var, value| host.add_env_var(var, value) end # REMOVE POST BEAKER 3: backwards compatability, do some setup based upon the global type # this is the worst and i hate it if host[:type] case host[:type] when /git|foss|aio/ Class.new.extend(Beaker::DSL).configure_foss_defaults_on(host) when /pe/ Class.new.extend(Beaker::DSL).configure_pe_defaults_on(host) end end #close the host to re-establish the connection with the new sshd settings host.close # print out the working env if host.is_powershell? host.exec(Command.new("SET")) else host.exec(Command.new("cat #{host[:ssh_env_file]}")) end end end |
#set_etc_hosts(host, etc_hosts) ⇒ Object
Append the provided string to the /etc/hosts file of the provided host
300 301 302 303 304 305 306 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 300 def set_etc_hosts(host, etc_hosts) if host['platform'] =~ /freebsd/ host.echo_to_file(etc_hosts, '/etc/hosts') else host.exec(Command.new("echo '#{etc_hosts}' > /etc/hosts")) end end |
#sync_root_keys(host, opts) ⇒ Object
Install a set of authorized keys using ROOT_KEYS_SCRIPT. This is a convenience method to allow for easy login to hosts after they have been provisioned with Beaker.
135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 135 def sync_root_keys host, opts # JJM This step runs on every system under test right now. We're anticipating # issues on Windows and maybe Solaris. We will likely need to filter this step # but we're deliberately taking the approach of "assume it will work, fix it # when reality dictates otherwise" logger = opts[:logger] block_on host do |host| logger.notify "Sync root authorized_keys from github on #{host.name}" # Allow all exit code, as this operation is unlikely to cause problems if it fails. if host['platform'] =~ /solaris|eos/ host.exec(Command.new(ROOT_KEYS_SYNC_CMD % "bash"), :accept_all_exit_codes => true) else host.exec(Command.new(ROOT_KEYS_SYNC_CMD % "env PATH=/usr/gnu/bin:$PATH bash"), :accept_all_exit_codes => true) end end rescue => e report_and_raise(logger, e, "sync_root_keys") end |
#timesync(host, opts) ⇒ Object
Run timesync on the provided hosts
35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 35 def timesync host, opts logger = opts[:logger] ntp_server = opts[:ntp_server] ? opts[:ntp_server] : NTPSERVER block_on host do |host| logger.notify "Update system time sync for '#{host.name}'" if host['platform'].include? 'windows' # The exit code of 5 is for Windows 2008 systems where the w32tm /register command # is not actually necessary. host.exec(Command.new("w32tm /register"), :acceptable_exit_codes => [0,5]) host.exec(Command.new("net start w32time"), :acceptable_exit_codes => [0,2]) host.exec(Command.new("w32tm /config /manualpeerlist:#{ntp_server} /syncfromflags:manual /update")) host.exec(Command.new("w32tm /resync")) logger.notify "NTP date succeeded on #{host}" else case when host['platform'] =~ /sles-/ ntp_command = "sntp #{ntp_server}" else ntp_command = "ntpdate -t 20 #{ntp_server}" end success=false try = 0 until try >= TRIES do try += 1 if host.exec(Command.new(ntp_command), :accept_all_exit_codes => true).exit_code == 0 success=true break end sleep SLEEPWAIT end if success logger.notify "NTP date succeeded on #{host} after #{try} tries" else raise "NTP date was not successful after #{try} tries" end end end rescue => e report_and_raise(logger, e, "timesync (--ntp)") end |
#validate_host(host, opts) ⇒ Object
Validate that hosts are prepared to be used as SUTs, if packages are missing attempt to install them.
Verifies the presence of #UNIX_PACKAGES on unix platform hosts, SLES_PACKAGES on SUSE platform hosts, DEBIAN_PACKAGES on debian platform hosts, CUMULUS_PACKAGES on cumulus platform hosts, WINDOWS_PACKAGES on cygwin-installed windows platform hosts, and PSWINDOWS_PACKAGES on non-cygwin windows platform hosts.
89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 |
# File 'lib/beaker/host_prebuilt_steps.rb', line 89 def validate_host host, opts logger = opts[:logger] block_on host do |host| case when host['platform'] =~ /sles-10/ check_and_install_packages_if_needed(host, SLES10_PACKAGES) when host['platform'] =~ /sles-/ check_and_install_packages_if_needed(host, SLES_PACKAGES) when host['platform'] =~ /debian/ check_and_install_packages_if_needed(host, DEBIAN_PACKAGES) when host['platform'] =~ /cumulus/ check_and_install_packages_if_needed(host, CUMULUS_PACKAGES) when (host['platform'] =~ /windows/ and host.is_cygwin?) check_and_install_packages_if_needed(host, WINDOWS_PACKAGES) when (host['platform'] =~ /windows/ and not host.is_cygwin?) check_and_install_packages_if_needed(host, PSWINDOWS_PACKAGES) when host['platform'] =~ /freebsd/ check_and_install_packages_if_needed(host, FREEBSD_PACKAGES) when host['platform'] =~ /openbsd/ check_and_install_packages_if_needed(host, OPENBSD_PACKAGES) when host['platform'] !~ /debian|aix|solaris|windows|sles-|osx-|cumulus|f5-/ check_and_install_packages_if_needed(host, UNIX_PACKAGES) end end rescue => e report_and_raise(logger, e, "validate") end |