Class: Beaker::SshConnection
- Inherits:
-
Object
- Object
- Beaker::SshConnection
- Defined in:
- lib/beaker/ssh_connection.rb
Constant Summary collapse
- RETRYABLE_EXCEPTIONS =
[ SocketError, Timeout::Error, Errno::ETIMEDOUT, Errno::EHOSTDOWN, Errno::EHOSTUNREACH, Errno::ECONNREFUSED, Errno::ECONNRESET, Errno::ENETUNREACH, Net::SSH::Disconnect, Net::SSH::AuthenticationFailed, IOError, ]
Instance Attribute Summary collapse
-
#logger ⇒ Object
Returns the value of attribute logger.
Class Method Summary collapse
Instance Method Summary collapse
-
#close ⇒ Object
closes this SshConnection.
-
#connect ⇒ Object
connect to the host.
- #execute(command, options = {}, stdout_callback = nil, stderr_callback = stdout_callback) ⇒ Object
-
#initialize(hostname, user = nil, ssh_opts = {}, options = {}) ⇒ SshConnection
constructor
A new instance of SshConnection.
- #process_stdin_for(channel, stdin) ⇒ Object
- #register_exit_code_for(channel, output) ⇒ Object
- #register_stderr_for(channel, output, callback = nil) ⇒ Object
- #register_stdout_for(channel, output, callback = nil) ⇒ Object
- #request_terminal_for(channel, command) ⇒ Object
- #scp_from(source, target, options = {}, dry_run = false) ⇒ Object
- #scp_to(source, target, options = {}, dry_run = false) ⇒ Object
- #try_to_execute(command, options = {}, stdout_callback = nil, stderr_callback = stdout_callback) ⇒ Object
-
#wait_for_connection_failure ⇒ Object
We expect the connection to close so wait for that to happen.
Constructor Details
#initialize(hostname, user = nil, ssh_opts = {}, options = {}) ⇒ SshConnection
Returns a new instance of SshConnection.
24 25 26 27 28 29 30 |
# File 'lib/beaker/ssh_connection.rb', line 24 def initialize hostname, user = nil, ssh_opts = {}, = {} @hostname = hostname @user = user @ssh_opts = ssh_opts @logger = [:logger] @options = end |
Instance Attribute Details
#logger ⇒ Object
Returns the value of attribute logger.
8 9 10 |
# File 'lib/beaker/ssh_connection.rb', line 8 def logger @logger end |
Class Method Details
.connect(hostname, user = 'root', ssh_opts = {}, options = {}) ⇒ Object
32 33 34 35 36 |
# File 'lib/beaker/ssh_connection.rb', line 32 def self.connect hostname, user = 'root', ssh_opts = {}, = {} connection = new hostname, user, ssh_opts, connection.connect connection end |
Instance Method Details
#close ⇒ Object
closes this SshConnection
62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 |
# File 'lib/beaker/ssh_connection.rb', line 62 def close begin if @ssh and not @ssh.closed? @ssh.close else @logger.warn("ssh.close: connection is already closed, no action needed") end rescue *RETRYABLE_EXCEPTIONS => e @logger.warn "Attemped ssh.close, (caught #{e.class.name} - #{e.})." rescue => e @logger.warn "ssh.close threw unexpected Error: #{e.class.name} - #{e.}. Shutting down, and re-raising error below" @ssh.shutdown! raise e ensure @ssh = nil @logger.warn("ssh connection to #{@hostname} has been terminated") end end |
#connect ⇒ Object
connect to the host
39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/beaker/ssh_connection.rb', line 39 def connect try = 1 last_wait = 0 wait = 1 @ssh ||= begin @logger.debug "Attempting ssh connection to #{@hostname}, user: #{@user}, opts: #{@ssh_opts}" Net::SSH.start(@hostname, @user, @ssh_opts) rescue *RETRYABLE_EXCEPTIONS => e if try <= 11 @logger.warn "Try #{try} -- Host #{@hostname} unreachable: #{e.class.name} - #{e.}" @logger.warn "Trying again in #{wait} seconds" sleep wait (last_wait, wait) = wait, last_wait + wait try += 1 retry else @logger.error "Failed to connect to #{@hostname}" raise end end end |
#execute(command, options = {}, stdout_callback = nil, stderr_callback = stdout_callback) ⇒ Object
145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 |
# File 'lib/beaker/ssh_connection.rb', line 145 def execute command, = {}, stdout_callback = nil, stderr_callback = stdout_callback try = 1 wait = 1 last_wait = 0 begin # ensure that we have a current connection object connect result = try_to_execute(command, , stdout_callback, stderr_callback) rescue *RETRYABLE_EXCEPTIONS => e if try < 11 sleep wait (last_wait, wait) = wait, last_wait + wait try += 1 @logger.error "Command execution '#{@hostname}$ #{command}' failed (#{e.class.name} - #{e.})" close @logger.debug "Preparing to retry: closed ssh object" retry else raise end end result end |
#process_stdin_for(channel, stdin) ⇒ Object
206 207 208 209 210 211 212 213 214 |
# File 'lib/beaker/ssh_connection.rb', line 206 def process_stdin_for channel, stdin # queue stdin data, force it to packets, and signal eof: this # triggers action in many remote commands, notably including # 'puppet apply'. It must be sent at some point before the rest # of the action. channel.send_data stdin.to_s channel.process channel.eof! end |
#register_exit_code_for(channel, output) ⇒ Object
200 201 202 203 204 |
# File 'lib/beaker/ssh_connection.rb', line 200 def register_exit_code_for channel, output channel.on_request("exit-status") do |ch, data| output.exit_code = data.read_long end end |
#register_stderr_for(channel, output, callback = nil) ⇒ Object
190 191 192 193 194 195 196 197 198 |
# File 'lib/beaker/ssh_connection.rb', line 190 def register_stderr_for channel, output, callback = nil channel.on_extended_data do |ch, type, data| if type == 1 callback[data] if callback output.stderr << data output.output << data end end end |
#register_stdout_for(channel, output, callback = nil) ⇒ Object
182 183 184 185 186 187 188 |
# File 'lib/beaker/ssh_connection.rb', line 182 def register_stdout_for channel, output, callback = nil channel.on_data do |ch, data| callback[data] if callback output.stdout << data output.output << data end end |
#request_terminal_for(channel, command) ⇒ Object
171 172 173 174 175 176 177 178 179 180 |
# File 'lib/beaker/ssh_connection.rb', line 171 def request_terminal_for channel, command channel.request_pty do |ch, success| if success @logger.debug "Allocated a PTY on #{@hostname} for #{command.inspect}" else abort "FAILED: could not allocate a pty when requested on " + "#{@hostname} for #{command.inspect}" end end end |
#scp_from(source, target, options = {}, dry_run = false) ⇒ Object
241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 |
# File 'lib/beaker/ssh_connection.rb', line 241 def scp_from source, target, = {}, dry_run = false return if dry_run local_opts = .dup if local_opts[:recursive].nil? local_opts[:recursive] = true end local_opts[:chunk_size] ||= 16384 result = Result.new(@hostname, [source, target]) result.stdout = "\n" @ssh.scp.download! source, target, local_opts do |ch, name, sent, total| result.stdout << "\tcopying %s: %10d/%d\n" % [name, sent, total] end # Setting these values allows reporting via result.log(test_name) result.stdout << " SCP'ed file #{@hostname}:#{source} to #{target}" # Net::Scp always returns 0, so just set the return code to 0. result.exit_code = 0 result.finalize! result end |
#scp_to(source, target, options = {}, dry_run = false) ⇒ Object
216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 |
# File 'lib/beaker/ssh_connection.rb', line 216 def scp_to source, target, = {}, dry_run = false return if dry_run local_opts = .dup if local_opts[:recursive].nil? local_opts[:recursive] = File.directory?(source) end local_opts[:chunk_size] ||= 16384 result = Result.new(@hostname, [source, target]) result.stdout = "\n" @ssh.scp.upload! source, target, local_opts do |ch, name, sent, total| result.stdout << "\tcopying %s: %10d/%d\n" % [name, sent, total] end # Setting these values allows reporting via result.log(test_name) result.stdout << " SCP'ed file #{source} to #{@hostname}:#{target}" # Net::Scp always returns 0, so just set the return code to 0. result.exit_code = 0 result.finalize! return result end |
#try_to_execute(command, options = {}, stdout_callback = nil, stderr_callback = stdout_callback) ⇒ Object
107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 |
# File 'lib/beaker/ssh_connection.rb', line 107 def try_to_execute command, = {}, stdout_callback = nil, stderr_callback = stdout_callback result = Result.new(@hostname, command) # why are we getting to this point on a dry run anyways? # also... the host creates connections through the class method, # which automatically connects, so you can't do a dry run unless you also # can connect to your hosts? return result if [:dry_run] @ssh.open_channel do |channel| request_terminal_for( channel, command ) if [:pty] channel.exec(command) do |terminal, success| abort "FAILED: to execute command on a new channel on #{@hostname}" unless success register_stdout_for terminal, result, stdout_callback register_stderr_for terminal, result, stderr_callback register_exit_code_for terminal, result process_stdin_for( terminal, [:stdin] ) if [:stdin] end end # Process SSH activity until we stop doing that - which is when our # channel is finished with... begin @ssh.loop rescue *RETRYABLE_EXCEPTIONS => e # this would indicate that the connection failed post execution, since the channel exec was successful @logger.warn "ssh channel on #{@hostname} received exception post command execution #{e.class.name} - #{e.}" close end result.finalize! @logger.last_result = result result end |
#wait_for_connection_failure ⇒ Object
We expect the connection to close so wait for that to happen
82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 |
# File 'lib/beaker/ssh_connection.rb', line 82 def wait_for_connection_failure try = 1 last_wait = 0 wait = 1 while try < 11 begin @logger.debug "Waiting for connection failure on #{@hostname} (attempt #{try}, try again in #{wait} second(s))" @ssh.open_channel do |channel| channel.exec('') #Just send something down the pipe end loop_tries = 0 #loop is actually loop_forver, so let it try 3 times and then quit instead of endless blocking @ssh.loop { loop_tries += 1 ; loop_tries < 4 } rescue *RETRYABLE_EXCEPTIONS => e @logger.debug "Connection on #{@hostname} failed as expected (#{e.class.name} - #{e.})" close #this connection is bad, shut it down return true end sleep wait (last_wait, wait) = wait, last_wait + wait try += 1 end false end |